/**
 * This class is a modified version of Mozilla's shell:
 * http://mxr.mozilla.org/mozilla/source/js/rhino/examples/Shell.java
 */
package com.objectpartners.envjs;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collection;

import junit.framework.Assert;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.JavaScriptException;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.WrappedException;

/**
 * @author Jeff Hemminger
 *
 */
public class Shell extends ScriptableObject {

    private boolean quitting;
    private static final long serialVersionUID = -5638074146250193112L;
    /**
     * Logger for this class.
     */
    private static final Logger logger = Logger.getLogger(Shell.class);

    @Override
    public String getClassName() {
        return "global";
    }

    /**
     * Print the string values of its arguments.
     *
     * This method is defined as a JavaScript function.
     * Note that its arguments are of the "varargs" form, which
     * allows it to handle an arbitrary number of arguments
     * supplied to the JavaScript function.
     *
     */
    public static void info(final Context cx, final Scriptable thisObj, final Object[] args, final Function funObj) {

    	for (int i = 0; i < args.length; i++) {
        
    		if (i > 0) {
                logger.info(" ");
            }

            logger.info(Context.toString(args[i]));
        }
    }

    /**
     * Print the string values of its arguments.
     *
     * This method is defined as a JavaScript function.
     * Note that its arguments are of the "varargs" form, which
     * allows it to handle an arbitrary number of arguments
     * supplied to the JavaScript function.
     *
     */
    public static void debug(final Context cx, final Scriptable thisObj, final Object[] args, final Function funObj) {
        
    	for (int i = 0; i < args.length; i++) {

        	if (i > 0) {
                logger.debug(" ");
            }

            logger.debug(Context.toString(args[i]));
        }
    }

    /**
     * Print the string values of its arguments.
     *
     * This method is defined as a JavaScript function.
     * Note that its arguments are of the "varargs" form, which
     * allows it to handle an arbitrary number of arguments
     * supplied to the JavaScript function.
     *
     */
    public static void error(final Context cx, final Scriptable thisObj, final Object[] args, final Function funObj) {
        
    	for (int i = 0; i < args.length; i++) {

        	if (i > 0) {
                logger.error(" ");
            }

            logger.error(Context.toString(args[i]));
        }
    }

    /**
     * Quit the shell.
     *
     * This only affects the interactive mode.
     *
     * This method is defined as a JavaScript function.
     */
    public void quit() {
        quitting = true;
    }

    /**
     * Get and set the language version.
     *
     * This method is defined as a JavaScript function.
     */
    public static double version(final Context cx, final Scriptable thisObj,
            final Object[] args, final Function funObj) {
        double result = cx.getLanguageVersion();
        if (args.length > 0) {
            double d = Context.toNumber(args[0]);
            cx.setLanguageVersion((int) d);
        }
        return result;
    }

    /**
     * Load and execute a set of JavaScript source files.
     *
     * This method is defined as a JavaScript function.
     *
     */
    public static void load(final Context cx, final Scriptable thisObj,
            final Object[] args, final Function funObj) {
        Shell shell = (Shell) getTopLevelScope(thisObj);
        for (int i = 0; i < args.length; i++) {
            shell.processSource(cx, Context.toString(args[i]));
        }
    }

    /**
     * Load and execute a set of JavaScript source files.
     *
     * This method is defined as a JavaScript function.
     *
     */
    public static void loadAllInDir(final Context cx, final Scriptable thisObj,
            final Object[] args, final Function funObj) {
        Shell shell = (Shell) getTopLevelScope(thisObj);
        String[] extensions = {"js"};
        for (int i = 0; i < args.length; i++) {
            Collection<File> coreCollection = FileUtils.listFiles(new File((String) args[i]), extensions, false);
            for (File file : coreCollection) {
              shell.processSource(cx, file.getAbsolutePath());
            }
        }
    }

    /**
     *
     * Asserts that two String values are equal.
     *
     * @param expected
     * @param actual
     */
    public static void assertEquals(final String expected, final String actual) {
        Assert.assertEquals(expected, actual);
    }

    /**
     *
     * Asserts that a given object is not null, and not of type Undefined.
     *
     * @param object
     */
    public static void assertNotNull(final Object object) {
        Assert.assertTrue("Provided object is undefined", !(object instanceof org.mozilla.javascript.Undefined));
        Assert.assertTrue("Provided object is null", object != null);
    }

    /**
     *
     * Calls junit's assertTrue.
     *
     * @param condition
     */
    public static void assertTrue(final boolean condition) {
        Assert.assertTrue(condition);
    }

    /**
     * Calls junit's assertFalse.
     *
     * @param condition
     */
    public static void assertFalse(final boolean condition) {
        Assert.assertFalse(condition);
    }


    /**
     * Evaluate JavaScript source.
     *
     * @param cx the current context
     * @param fileName the name of the file to compile, or null
     */
    public Object processSource(final Context cx, final String fileName) {
        try {
            FileReader in = new FileReader(fileName);
            return doSourceProcess(cx, in, fileName);
        } catch (FileNotFoundException ex) {
            Context.reportError("Couldn't open file \"" + fileName + "\".");
            return null;
        }
    }

    /**
     *
     * The <code>processSource</code> ...
     * Processes the provided file as javascript.
     * @param cx
     * @param file
     * @return
     */
    public Object processSource(final Context cx, final File file) {
        try {
            FileReader in = new FileReader(file);
            return doSourceProcess(cx, in, file.getAbsolutePath());
        } catch (FileNotFoundException ex) {
            Context.reportError("Couldn't open file \"" + file.getAbsolutePath() + "\".");
            return null;
        }
    }

    private Object doSourceProcess(final Context cx, final FileReader reader, final String fileName) {
        Object result = null;
        try {
            // Here we evalute the entire contents of the file as
            // a script. Text is printed only if the print() function
            // is called.
            synchronized (cx) {
                result = cx.evaluateReader(this, reader, fileName, 1, null);
            }
        } catch (WrappedException we) {
            logger.error(we.getWrappedException().toString());
            logger.error(we.fillInStackTrace());
        } catch (EvaluatorException ee) {
            logger.error("js: " + ee.getMessage());
        } catch (JavaScriptException jse) {
            logger.error("js: " + jse.getMessage());
        } catch (IOException ioe) {
            logger.error(ioe.toString());
            logger.error(ioe.fillInStackTrace());
        } finally {
            try {
                reader.close();
            } catch (IOException ioe) {
                logger.error(ioe.toString());
                logger.error(ioe.fillInStackTrace());
            }
        }
        return result;
    }
}
